## Twisted Life: A game that uses a variant John H. Conway's Life cellular
## automaton in which Life forms twist around the center of the universe.
## Copyright (C) 2007  Paul Reiners
##
## This program is free software: you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation, either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.

import random

class TwistedLife:
    def __init__(self, radius = 2, universe = None):
        self.radius = radius
        size = 2 * radius + 1
        self.neighbors = ([[-1, -1], [-1, 0], [-1, 1],
                           [0, -1],           [0, 1],
                           [1, -1],  [1, 0],  [1, 1]])
        if universe == None:
            self.universe = []
            for i in range(size):
                row = []
                for j in range(size):
                    if random.random() < 0.125:
                        row.append(1)
                    else:
                        row.append(0)
                self.universe.append(row)
        else:
            self.universe = universe

    def __repr__(self):
        result = ''
        for i in range(len(self.universe)):
            for j in range(len(self.universe[i])):
                result += str(self.universe[i][j])
            result += '\n'

        return result

    def nextGen(self):
        newGen = []
        height = len(self.universe)
        width = len(self.universe[0])
        for i in range(height):
            newRow = []
            for j in range(width):
                count = 0
                for k in range(len(self.neighbors)):
                    neighbor = self.neighbors[k]
                    y = (i + neighbor[0]) % height
                    x = (j + neighbor[1]) % width
                    if self.universe[y][x]:
                        count += 1
                oldVal = self.universe[i][j]
                if count < 2 or count > 3:
                    newRow.append(0)
                elif oldVal == 0 and count == 3:
                    newRow.append(1)
                else:
                    newRow.append(oldVal)
            newGen.append(newRow)

        # Now twist
        twisted = []
        for i in range(height):
            row = [0 for j in range(width)]
            twisted.append(row)
        center = self.radius
        for i in range(-self.radius, self.radius + 1):
            for j in range(-self.radius, self.radius + 1):
                absI = abs(i)
                absJ = abs(j)
                dist = max(absI, absJ)
                if i == 0 and j == 0:
                    iOffSet, jOffSet = 0, 0
                elif dist == absI and i < 0 and not (i == -self.radius and j == -self.radius):
                    iOffSet, jOffSet = 0, -1
                elif dist == absJ and j > 0:
                    iOffSet, jOffSet = -1, 0
                elif dist == absI and i > 0:
                    iOffSet, jOffSet = 0, 1
                else:
                    iOffSet, jOffSet = 1, 0
                twisted[i + self.radius][j + self.radius] = newGen[i + self.radius + iOffSet][j + self.radius + jOffSet]
        
        self.universe = twisted

    def isAlive(self, x, y):
        if self.universe[x][y]:
            return True
        else:
            return False

    def diameter(self):
        return len(self.universe)

    def getCenter(self):
        return (self.radius, self.radius)

if __name__ == '__main__':
    universe = TwistedLife()
    print universe
    universe.nextGen()
    print universe
    universe.nextGen()
    print universe

    python =   TwistedLife(universe=[[0,0,0,0,0,0,0],
                 [0,0,0,0,0,0,0],
                 [0,0,0,0,1,1,0],
                 [0,1,0,1,0,1,0],
                 [0,1,1,0,0,0,0],
                 [0,0,0,0,0,0,0]])
    print python
    python.nextGen()
    print python
    python.nextGen()
    print python
    python.nextGen()
    print python
